#include "glview.h"

GLView::GLView(Scene *scene,bool perspective)
{
    this->persp = perspective;
    this->scene = scene;

    setHome(new QQuaternion(), new QVector3D(0.0,0.0,-3.0));
}

QSize GLView::minimumSizeHint() const
{
    return QSize(50, 50);
}

QSize GLView::sizeHint() const
{
    return QSize(600, 400);
}


void GLView::initializeGL ( ) {
    glClearColor(0.0, 0.0, 0.0, 0.0);
    glEnable(GL_DEPTH_TEST);


    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
    static GLfloat lightPosition[4] = { 0.5, 0.0, 2.0, 1.0 };
    glLightfv(GL_LIGHT0, GL_POSITION, lightPosition);

    //Shader Setup
    initShader();
    shader->bind();
}

void GLView::initShader()
{
    QGLShader *vertex = new QGLShader(QGLShader::Vertex);
    if(!vertex->compileSourceFile(QLatin1String(":/phong.vert")))
        qCritical()<< "Vertex Shader failed"<< vertex->log();

    QGLShader *fragment = new QGLShader(QGLShader::Fragment);
    if(!fragment->compileSourceFile(QLatin1String(":/phong.frag")))
        qCritical()<< "Fragment Shader failed"<< fragment->log();

    shader = new QGLShaderProgram(this);
    shader->addShader(vertex);
    shader->addShader(fragment);
    if(!shader->link())
        qCritical()<< "Linking  failed"<<shader->log();
}


void GLView::setHome(QQuaternion *rotation, QVector3D *translation)
{
    //qDebug()<<*rotation<<" trans:"<<*translation;
    this->homeRotation = rotation;
    this->homeTranslation = translation;
    home();

}

void GLView::home()
{
    this->rotation = homeRotation;
    this->translation = homeTranslation;
}

void GLView::paintGL ()
{

    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    //set Projection
    resizeGL(0,0);
    glLoadIdentity();

    //Camera Tranforations
    QMatrix4x4 mat = QMatrix4x4();
    mat.translate(*translation);
    glMultMatrixf(mat.data());

   // glRotated(45,0,1,0);

    mat = QMatrix4x4();
    mat.rotate(*rotation);
    glMultMatrixf(mat.data());

    //draw Scene
    scene->draw();

}
void GLView::resizeGL(int width , int height )
{
    glViewport(0, 0, this->width(), this->height());
    GLdouble aspect = 1.0*this->width()/this->height();

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    if(persp){
        perspective(45.0,aspect,0.01,100.0);
    } else {
        int size = 1;
        glOrtho(-size*aspect,size*aspect,-size,size,0.01,100.0);
    }
    glMatrixMode(GL_MODELVIEW);

}

void GLView::perspective(GLdouble fovy, GLdouble aspect, GLdouble zNear, GLdouble zFar)
{
    GLdouble xmin, xmax, ymin, ymax;

    ymax = zNear * tan( fovy * M_PI / 360.0 );
    ymin = -ymax;
    xmin = ymin * aspect;
    xmax = ymax * aspect;

    glFrustum( xmin, xmax, ymin, ymax, zNear, zFar );
}



void GLView::mousePressEvent(QMouseEvent *event ) {}
void GLView::mouseMoveEvent(QMouseEvent *event ){}
void GLView::wheelEvent(QWheelEvent *event ) {}

