#include "mainwindow.h"



MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent)
{

    setWindowTitle("Hello Cube");

    menuBar = new QMenuBar();
    toolBar = new QToolBar("Shading",this);
    statusBar = new QStatusBar(this);

    //Views
    scene = new Scene();

    perspectiveView = new GLView(scene,true);
    frontView = new GLView(scene,false);
    frontView->setHome(new QQuaternion(), new QVector3D(0.0,0.0,-3.0));
    leftView = new GLView(scene,false);
    leftView->setHome(new QQuaternion(QQuaternion::fromAxisAndAngle(0.0,1.0,0.0,90.0).toVector4D()), new QVector3D(0.0,0.0,-3.0));
    topView = new GLView(scene,false);
    topView->setHome(new QQuaternion(QQuaternion::fromAxisAndAngle(1.0,0.0,0.0,90.0).toVector4D()), new QVector3D(0.0,0.0,-3.0));

    topSplit = new QSplitter(Qt::Horizontal,this);
    bottomSplit = new QSplitter(Qt::Horizontal,this);
    verticalSplit = new QSplitter(Qt::Vertical,this);

    activeView = perspectiveView;

    topSplit->addWidget(perspectiveView);
    topSplit->addWidget(frontView);

    bottomSplit->addWidget(leftView);
    bottomSplit->addWidget(topView);

    verticalSplit->addWidget(topSplit);
    verticalSplit->addWidget(bottomSplit);
    setCentralWidget(verticalSplit);

    showQuad();
    //showSingle();
    //showDual();
    //

    //File Menu Actions
    fileMenu = new QMenu("&File");
    viewGoup = new QActionGroup(this);
    viewMenu = new QMenu("View",this);

    interactionGroup = new QActionGroup(this);
    interactionMenu = new QMenu("Interaction",this);


    exitAction = new QAction("E&xit",fileMenu);
    exitAction->setShortcut(QKeySequence::Quit);
    connect(exitAction, SIGNAL(triggered()), this, SLOT(close()));


    //Interaction Actions
    cameraMode = new QAction("Camera",this);
    cameraMode->setCheckable(true);
    cameraMode->setIcon(QIcon(":/img/camera.png"));
    cameraMode->setChecked(true);
    // connect(cameraMode, SIGNAL(triggered()), mainWidget, SLOT(showWireframe()));
    interactionGroup->addAction(cameraMode);

    editMode = new QAction("Edit",this);
    editMode->setCheckable(true);
    editMode->setIcon(QIcon(":/img/select.png"));
    // connect(editMode, SIGNAL(triggered()), mainWidget, SLOT(showWireframe()));
    interactionGroup->addAction(editMode);

    //View Actions
    singleView = new QAction("Single View",this);
    singleView->setShortcut(QKeySequence("1"));
    singleView->setCheckable(true);

    singleView->setIcon(QIcon(":/img/view-single.png"));
    connect(singleView, SIGNAL(triggered()), this, SLOT(showSingle()));
    viewGoup->addAction(singleView);

    dualView = new QAction("Dual View",this);
    dualView->setShortcut(QKeySequence("2"));
    dualView->setCheckable(true);
    dualView->setIcon(QIcon(":/img/view-dual.png"));
    connect(dualView, SIGNAL(triggered()), this, SLOT(showDual()));
    viewGoup->addAction(dualView);

    quadView = new QAction("Quad View",this);
    quadView->setShortcut(QKeySequence("4"));
    quadView->setCheckable(true);
    quadView->setChecked(true);
    quadView->setIcon(QIcon(":/img/viewports.png"));

    connect(quadView, SIGNAL(triggered()), this, SLOT(showQuad()));
    viewGoup->addAction(quadView);

    viewMenu->addAction(singleView);
    viewMenu->addAction(dualView);
    viewMenu->addAction(quadView);


    //Other Actions
    aboutAction = new QAction("About",menuBar);
    connect(aboutAction,SIGNAL(triggered()),this,SLOT(showAboutBox()));

    camHome = new QAction(QIcon(":/img/cam_home.png"),"Cam Home", toolBar);
    connect(camHome, SIGNAL(triggered(bool)),activeView,SLOT(home()));


    // Assemble Menus
    fileMenu->addAction(exitAction);
    menuBar->addMenu(fileMenu);
    menuBar->addAction(aboutAction);

    setMenuBar(menuBar);

    //Assemble Tool Bar
    toolBar->addAction(cameraMode);
    toolBar->addAction(editMode);

    toolBar->addSeparator();
    toolBar->addAction(camHome);
    toolBar->addSeparator();

    QToolButton* toolButton = new QToolButton();
    toolButton->setIcon(QIcon(":/img/viewports.png"));
    toolButton->setMenu(viewMenu);
    toolButton->setPopupMode(QToolButton::InstantPopup);

    QWidgetAction* toolButtonAction = new QWidgetAction(this);
    toolButtonAction->setDefaultWidget(toolButton);

    toolBar->addAction(toolButtonAction);

    addToolBar( toolBar);

    statusBar->showMessage("Hello");
    setStatusBar(statusBar);


}

void MainWindow::showSingle(){
   perspectiveView->show();
   frontView->hide();
   leftView->hide();
   topView->hide();
}
void MainWindow::showDual(){

    perspectiveView->show();
    frontView->show();
    leftView->hide();
    topView->hide();
}

void MainWindow::showQuad(){

    perspectiveView->show();
    frontView->show();
    leftView->show();
    topView->show();


}



MainWindow::~MainWindow()
{

}

void MainWindow::showAboutBox()
{
    QMessageBox msgBox;
    msgBox.setWindowTitle("About Hello Cube!");
    msgBox.setText("Written by Kai Westerkamp");
    msgBox.exec();
}
